@@ -19,7 +19,7 @@ use self::core::ops::Not;
 
 // TODO better docs and tests
 
-/// Methods for working with `u8`s UTF-8.
+/// Methods for working with `u8`s as UTF-8 bytes.
 pub trait U8UtfExt {
     /// How many more bytes will you need to complete this codepoint?
     ///
@@ -57,24 +57,27 @@ impl U8UtfExt for u8 {
 pub trait U16UtfExt {
     /// Will you need an extra unit to complete this codepoint?
     ///
-    /// # Failures:
-    ///
-    /// 0xdc00..0xe000
-    fn utf16_needs_extra_unit(self) -> Option<bool>;
+    /// Returns `Err` for trailing surrogates, `Ok(true)` for leading surrogates,
+    /// and `Ok(false)` for others.
+    fn utf16_needs_extra_unit(self) -> Result<bool,InvalidUtf16FirstUnit>;
 
     /// Does this `u16` need another `u16` to complete a codepoint?
     /// Returns `(self & 0xfc00) == 0xd800`
-    fn utf16_is_leading_surrogate(self) -> bool;
+    ///
+    /// Is basically an unchecked variant of `utf16_needs_extra_unit()`.
+    fn is_utf16_leading_surrogate(self) -> bool;
 }
 impl U16UtfExt for u16 {
-    fn utf16_needs_extra_unit(self) -> Option<bool> {match self {
+    fn utf16_needs_extra_unit(self) -> Result<bool,InvalidUtf16FirstUnit> {match self {
         // https://en.wikipedia.org/wiki/UTF-16#U.2B10000_to_U.2B10FFFF
-        0x_dc_00...0x_df_ff => None,
-        0x_d8_00...0x_db_ff => Some(true),
-        _                   => Some(false),
+        0x00_00...0xd7_ff => Ok(false),
+        0xe0_00...0xff_ff => Ok(false),
+        0xd8_00...0xdb_ff => Ok(true),
+        0xdc_00...0xdf_ff => Err(InvalidUtf16FirstUnit),
+                _         => unreachable!()
     }}
 
-    fn utf16_is_leading_surrogate(self) -> bool {
+    fn is_utf16_leading_surrogate(self) -> bool {
         (self & 0xfc00) == 0xd800// Clear the ten content bytes of a surrogate,
                                  // and see if it's a leading surrogate.
     }
@@ -111,29 +114,6 @@ pub trait CharExt: Sized {
     fn iter_utf16_units(self) -> Utf16Iterator;
 
 
-    /// Convert this char to UTF-8, and then
-    /// returns the number of bytes written.
-    ///
-    /// # Panics
-    /// Will panic the buffer is too small;
-    /// You can get the required length from `.len_utf8()`,
-    /// but a buffer of length four is always large enough.
-    ///
-    /// A similar alternative to the proposed `char.write_utf8()`,
-    fn to_utf8_slice(self,  dst: &mut[u8]) -> usize;
-
-    /// Convert this char to UTF-16, and then
-    /// returns the number of units written.
-    ///
-    /// # Panics
-    /// Will panic the buffer is too small;
-    /// You can get the required length from `.len_utf16()`,
-    /// but a buffer of length two is always large enough.
-    ///
-    /// a similar alternative to the proposed `char.write_utf16()`,
-    fn to_utf16_slice(self,  dst: &mut[u16]) -> usize;
-
-
     /// Convert this char to an UTF-8 array and lenght,
     ///
     /// The returned array is left-aligned with unused bytes set to zero,
@@ -146,11 +126,13 @@ pub trait CharExt: Sized {
 
 
 
-    /// Create a `char` from the start of a slice intepreted as UTF-8, and return how many bytes were needed.
-    fn from_utf8_slice(src: &[u8]) -> Result<(Self,usize),InvalidUtf8Slice>;
+    /// Create a `char` from the start of a UTF-8 slice,
+    /// and also return how many bytes were used.
+    fn from_utf8_slice_start(src: &[u8]) -> Result<(Self,usize),InvalidUtf8Slice>;
 
-    /// Read one or two UTF-16 units into a `char`, and also return how many units were needed.
-    fn from_utf16_slice(src: &[u16]) -> Result<(Self,usize), InvalidUtf16Slice>;
+    /// Create a `char` from the start of a UTF-16 slice,
+    /// and also return how many units were used.
+    fn from_utf16_slice_start(src: &[u16]) -> Result<(Self,usize), InvalidUtf16Slice>;
 
 
     /// Convert an UTF-8 sequence as returned from `.to_utf8_array()` into a `char`
@@ -193,9 +175,6 @@ impl CharExt for char {
     fn iter_utf8_bytes(self) -> Utf8Iterator {
         self.to_utf8().into_iter()
     }
-    fn to_utf8_slice(self,  dst: &mut[u8]) -> usize {
-        self.to_utf8().to_slice(dst)
-    }
 
     fn to_utf8_array(self) -> ([u8; 4], usize) {
         let len = self.len_utf8();
@@ -210,7 +189,8 @@ impl CharExt for char {
             parts<<=8;  parts |= c & 0x3f;
             parts |= 0x80_80_80_80;// set the most significant bit
             parts >>= 8*(4-len);// right-align bytes
-            // Now, unused bytes are zero, (which matters for Utf8Char.eq()) and the rest are 0b10xx_xxxx
+            // Now, unused bytes are zero, (which matters for Utf8Char.eq())
+            // and the rest are 0b10xx_xxxx
 
             // set header on first byte
             parts |= (0xff_00u32 >> len)  &  0xff;// store length
@@ -222,26 +202,30 @@ impl CharExt for char {
     }
 
 
-    fn from_utf8_slice(src: &[u8]) -> Result<(Self,usize),InvalidUtf8Slice> {
+    fn from_utf8_slice_start(src: &[u8]) -> Result<(Self,usize),InvalidUtf8Slice> {
         use errors::InvalidUtf8::*;
         use errors::InvalidUtf8Slice::*;
-        let first = *try!(src.first().ok_or(TooShort(1)));
-        let src = match first.extra_utf8_bytes() {
+        let first = match src.first() {
+            Some(first) => *first,
+            None => return Err(TooShort(1)),
+        };
+        let bytes = match first.extra_utf8_bytes() {
             Err(e)    => return Err(Utf8(FirstByte(e))),
             Ok(0)     => return Ok((first as char, 1)),
             Ok(extra) if extra >= src.len()
                       => return Err(TooShort(extra+1)),
             Ok(extra) => &src[..extra+1],
         };
-        if let Some(i) = src.iter().skip(1).position(|&b| (b >> 6) != 0b10 ) {
+        if let Some(i) = bytes.iter().skip(1).position(|&b| (b >> 6) != 0b10 ) {
             Err(Utf8(NotAContinuationByte(i+1)))
-        } else if overlong(src[0], src[1]) {
+        } else if overlong(bytes[0], bytes[1]) {
             Err(Utf8(OverLong))
         } else {
-            let c = unsafe{ char::from_utf8_exact_slice_unchecked(src) };
-            char::from_u32_detailed(c as u32)
-                .map(|c| (c,src.len()) )
-                .map_err(|e| Codepoint(e) )
+            let c = unsafe{ char::from_utf8_exact_slice_unchecked(bytes) };
+            match char::from_u32_detailed(c as u32) {
+                Ok(c) => Ok((c, bytes.len())),
+                Err(e) => Err(Codepoint(e)),
+            }
         }
     }
 
@@ -289,10 +273,6 @@ impl CharExt for char {
         self.to_utf16().into_iter()
     }
 
-    fn to_utf16_slice(self,  dst: &mut[u16]) -> usize {
-        self.to_utf16().to_slice(dst)
-    }
-
     fn to_utf16_tuple(self) -> (u16, Option<u16>) {
         let c = self as u32;
         if c <= 0x_ff_ff {// single (or reserved, which we ignore)
@@ -306,53 +286,54 @@ impl CharExt for char {
     }
 
 
-    fn from_utf16_slice(src: &[u16]) -> Result<(Self,usize), InvalidUtf16Slice> {
+    fn from_utf16_slice_start(src: &[u16]) -> Result<(Self,usize), InvalidUtf16Slice> {
         use errors::InvalidUtf16Slice::*;
-        let first = *try!(src.first().ok_or(EmptySlice));
-        match (first.utf16_needs_extra_unit(), src.get(1).cloned()) {
-            (Some(false),              _            )  =>  Ok((1, None)),
-            (Some(true) ,  Some(0x_dc_00...0x_df_ff))  =>  Ok((2, Some(src[1]))),
-            (Some(true) ,  Some(         _         ))  =>  Err(SecondNotLowSurrogate),
-            (Some(true) ,  None                     )  =>  Err(MissingSecond),
-            (None       ,              _            )  =>  Err(FirstLowSurrogate),
-        }.map(|(len,second)| (unsafe{ char::from_utf16_tuple_unchecked((first,second)) }, len) )
+        unsafe {match (src.get(0), src.get(1)) {
+            (Some(&u @ 0x00_00...0xd7_ff), _) |
+            (Some(&u @ 0xe0_00...0xff_ff), _)
+                => Ok((char::from_u32_unchecked(u as u32), 1)),
+            (Some(&0xdc_00...0xdf_ff), _) => Err(FirstLowSurrogate),
+            (None, _) => Err(EmptySlice),
+            (Some(&f @ 0xd8_00...0xdb_ff), Some(&s @ 0xdc_00...0xdf_ff))
+                => Ok((char::from_utf16_tuple_unchecked((f, Some(s))), 2)),
+            (Some(&0xd8_00...0xdb_ff), Some(_)) => Err(SecondNotLowSurrogate),
+            (Some(&0xd8_00...0xdb_ff), None) => Err(MissingSecond),
+            (Some(_), _) => unreachable!()
+        }}
     }
 
     fn from_utf16_tuple(utf16: (u16, Option<u16>)) -> Result<Self, InvalidUtf16Tuple> {
         use errors::InvalidUtf16Tuple::*;
-        match utf16 {
-            (0x_00_00...0x_d7_ff, None) => Ok(()),// single
-            (0x_e0_00...0x_ff_ff, None) => Ok(()),// single
-            (0x_d8_00...0x_db_ff, Some(0x_dc_00...0x_df_ff)) => Ok(()),// correct surrogate pair
-            (0x_d8_00...0x_db_ff, Some(_)) => Err(InvalidSecond),
-            (0x_d8_00...0x_db_ff, None) => Err(MissingSecond),
-            (0x_dc_00...0x_df_ff, _) => Err(FirstIsTrailingSurrogate),
-            (_, Some(_)) => Err(SuperfluousSecond),// should be no second
-            (_, _) => unreachable!()
-        }.map(|_| unsafe{ char::from_utf16_tuple_unchecked(utf16) } )
+        unsafe{ match utf16 {
+            (0x00_00...0xd7_ff, None) | // single
+            (0xe0_00...0xff_ff, None) | // single
+            (0xd8_00...0xdb_ff, Some(0xdc_00...0xdf_ff)) // correct surrogate
+                => Ok(char::from_utf16_tuple_unchecked(utf16)),
+            (0xd8_00...0xdb_ff, Some(_)) => Err(InvalidSecond),
+            (0xd8_00...0xdb_ff, None   ) => Err(MissingSecond),
+            (0xdc_00...0xdf_ff,    _   ) => Err(FirstIsTrailingSurrogate),
+            (        _        , Some(_)) => Err(SuperfluousSecond),
+            (        _        , None   ) => unreachable!()
+        }}
     }
 
     unsafe fn from_utf16_tuple_unchecked(utf16: (u16, Option<u16>)) -> Self {
-        let mut c = utf16.0 as u32;
-        if let Some(second) = utf16.1 {
-            let high = (c-0x_d8_00) << 10;
-            let low = second as u32 - 0x_dc_00;
-            c = high | low;
-            c += 0x_01_00_00;
+        match utf16.1 {
+            Some(second) => combine_surrogates(utf16.0, second),
+            None         => char::from_u32_unchecked(utf16.0 as u32)
         }
-        unsafe{ char::from_u32_unchecked(c) }
     }
 
 
     fn from_u32_detailed(c: u32) -> Result<Self,InvalidCodepoint> {
         use errors::InvalidCodepoint::*;
-        match c {
-            // reserved for UTF-16 surrogate pairs
-            0xd8_00...0xdf_ff => Err(Utf16Reserved),
-            // too big
-            0x11_00_00...u32::MAX => Err(TooHigh),
-            _ => Ok(unsafe{ char::from_u32_unchecked(c) }),
-        }
+        unsafe{ match c {
+            0x00_00_00...0x00_d7_ff => Ok(char::from_u32_unchecked(c)),
+            0x00_d8_00...0x00_df_ff => Err(Utf16Reserved),
+            0x00_e0_00...0x10_ff_ff => Ok(char::from_u32_unchecked(c)),
+            0x11_00_00...u32::MAX   => Err(TooHigh),
+                       _            => unreachable!()
+        }}
     }
 }
 
@@ -366,3 +347,11 @@ fn overlong(first: u8,  second: u8) -> bool {
     let both = both << 1+both.not().leading_zeros();
     both.leading_zeros() >= 5
 }
+
+// Create a `char` from a leading and a trailing surrogate.
+unsafe fn combine_surrogates(first: u16, second: u16) -> char {
+    let high = (first & 0x_03_ff) as u32;
+    let low = (second & 0x_03_ff) as u32;
+    let c = ((high << 10) | low) + 0x_01_00_00; // no, the constant can't be or'd in
+    char::from_u32_unchecked(c)
+}
