@@ -8,12 +8,16 @@
 
 use Utf16Iterator;
 use CharExt;
+use U16UtfExt;
+use Utf8Char;
 use errors::{InvalidUtf16Slice,InvalidUtf16Tuple};
 extern crate core;
 use self::core::{hash,fmt,cmp};
 use self::core::borrow::Borrow;
 use self::core::ops::Deref;
-#[cfg(not(feature="no_std"))]
+#[cfg(feature="std")]
+use self::core::iter::FromIterator;
+#[cfg(feature="std")]
 use std::ascii::AsciiExt;
 #[cfg(feature="ascii")]
 use self::core::char;
@@ -32,10 +36,9 @@ use self::ascii::{AsciiChar,ToAsciiChar,ToAsciiCharError};
 #[derive(Clone,Copy)]
 
 
-/// Store a `char` as UTF-16 so it can be borrowed as a slice
+/// An unicode codepoint stored as UTF-16.
 ///
-/// Size is identical to `char`.
-/// Cannot represent all 2^32-1 possible values, but can do all valid ones.
+/// It can be borrowed as an `u16` slice, and has the same size as `char`.
 pub struct Utf16Char {
     units: [u16; 2],
 }
@@ -51,6 +54,31 @@ impl From<char> for Utf16Char {
         Utf16Char{ units: [first, second.unwrap_or(0)] }
     }
 }
+impl From<Utf8Char> for Utf16Char {
+    fn from(utf8: Utf8Char) -> Utf16Char {
+        let (b, utf8_len) = utf8.to_array();
+        match utf8_len {
+            1 => Utf16Char{ units: [b[0] as u16, 0] },
+            4 => {// need surrogate
+                let mut first = 0xd800 - (0x01_00_00u32 >> 10) as u16;
+                first += (b[0] as u16 & 0x07) << 8;
+                first += (b[1] as u16 & 0x3f) << 2;
+                first += (b[2] as u16 & 0x30) >> 4;
+                let mut second = 0xdc00;
+                second |= (b[2] as u16 & 0x0f) << 6;
+                second |=  b[3] as u16 & 0x3f;
+                Utf16Char{ units: [first, second] }
+            },
+            _ => { // 2 or 3
+                let mut unit = ((b[0] as u16 & 0x1f) << 6) | (b[1] as u16 & 0x3f);
+                if utf8_len == 3 {
+                    unit = (unit << 6) | (b[2] as u16 & 0x3f);
+                }
+                Utf16Char{ units: [unit, 0] }
+            },
+        }
+    }
+}
 impl From<Utf16Char> for char {
     fn from(uc: Utf16Char) -> char {
         unsafe{ char::from_utf16_tuple_unchecked(uc.to_tuple()) }
@@ -64,6 +92,27 @@ impl IntoIterator for Utf16Char {
         Utf16Iterator::from(self)
     }
 }
+#[cfg(feature="std")]
+impl Extend<Utf16Char> for Vec<u16> {
+    fn extend<I:IntoIterator<Item=Utf16Char>>(&mut self, iter: I) {
+        let iter = iter.into_iter();
+        self.reserve(iter.size_hint().0);
+        for u16c in iter {
+            self.push(u16c.units[0]);
+            if u16c.units[1] != 0 {
+                self.push(u16c.units[1]);
+            }
+        }
+    }
+}
+#[cfg(feature="std")]
+impl FromIterator<Utf16Char> for Vec<u16> {
+    fn from_iter<I:IntoIterator<Item=Utf16Char>>(iter: I) -> Self {
+        let mut vec = Vec::new();
+        vec.extend(iter);
+        return vec;
+    }
+}
 
 
   /////////////////
@@ -90,22 +139,25 @@ impl Deref for Utf16Char {
   ////////////////
  //ascii traits//
 ////////////////
-#[cfg(not(feature="no_std"))]
+#[cfg(feature="std")]
 impl AsciiExt for Utf16Char {
     type Owned = Self;
     fn is_ascii(&self) -> bool {
         self.units[0] < 128
     }
     fn eq_ignore_ascii_case(&self,  other: &Self) -> bool {
-        self.to_char().eq_ignore_ascii_case(&other.to_char())
+        self.to_ascii_lowercase() == other.to_ascii_lowercase()
     }
     fn to_ascii_uppercase(&self) -> Self {
-        self.to_char().to_ascii_uppercase().to_utf16()
+        let n = self.units[0].wrapping_sub(b'a' as u16);
+        if n < 26 {Utf16Char{ units: [n+b'A' as u16, 0] }}
+        else      {*self}
     }
     fn to_ascii_lowercase(&self) -> Self {
-        self.to_char().to_ascii_lowercase().to_utf16()
+        let n = self.units[0].wrapping_sub(b'A' as u16);
+        if n < 26 {Utf16Char{ units: [n+b'a' as u16, 0] }}
+        else      {*self}
     }
-    // Theese methods are what prevents this from becoming stable
     fn make_ascii_uppercase(&mut self) {
         *self = self.to_ascii_uppercase()
     }
@@ -115,14 +167,14 @@ impl AsciiExt for Utf16Char {
 }
 
 #[cfg(feature="ascii")]
-/// Requires feature "ascii".
+/// Requires the feature "ascii".
 impl From<AsciiChar> for Utf16Char {
     fn from(ac: AsciiChar) -> Self {
         Utf16Char{ units: [ac.as_byte() as u16, 0] }
     }
 }
 #[cfg(feature="ascii")]
-/// Requires feature "ascii".
+/// Requires the feature "ascii".
 impl ToAsciiChar for Utf16Char {
     fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
         unsafe{ AsciiChar::from(char::from_u32_unchecked(self.units[0] as u32)) }
@@ -148,12 +200,21 @@ impl fmt::Debug for Utf16Char {
 }
 impl cmp::PartialOrd for Utf16Char {
     fn partial_cmp(&self,  rhs: &Self) -> Option<cmp::Ordering> {
-        self.to_char().partial_cmp(&rhs.to_char())
+        Some(self.cmp(rhs))
     }
 }
 impl cmp::Ord for Utf16Char {
     fn cmp(&self,  rhs: &Self) -> cmp::Ordering {
-        self.to_char().cmp(&rhs.to_char())
+        // Shift the first unit by 0xd if surrogate, and 0 otherwise.
+        // This ensures surrogates are always greater than 0xffff, and
+        // that the second unit only affect the result when the first are equal.
+        // Multiplying by a constant factor isn't enough because that factor
+        // would have to be greater than 1023 and smaller than 5.5.
+        let lhs = (self.units[0] as u32, self.units[1] as u32);
+        let rhs = (rhs.units[0] as u32, rhs.units[1] as u32);
+        let lhs = (lhs.0 << (lhs.1 >> 12)) + lhs.1;
+        let rhs = (rhs.0 << (rhs.1 >> 12)) + rhs.1;
+        lhs.cmp(&rhs)
     }
 }
 
@@ -164,23 +225,48 @@ impl cmp::Ord for Utf16Char {
 impl Utf16Char {
     /// Validate and store the first UTF-16 codepoint in the slice.
     /// Also return how many units were needed.
-    pub fn from_slice(src: &[u16]) -> Result<(Self,usize),InvalidUtf16Slice> {
-        char::from_utf16_slice(src).map(|(_,len)| {
+    pub fn from_slice_start(src: &[u16]) -> Result<(Self,usize), InvalidUtf16Slice> {
+        char::from_utf16_slice_start(src).map(|(_,len)| {
             let second = if len==2 {src[1]} else {0};
             (Utf16Char{ units: [src[0], second] }, len)
         })
     }
+    /// Store the first UTF-16 codepoint of the slice.
+    ///
+    /// # Safety
+    /// The slice must be non-empty and start with a valid UTF-16 codepoint.  
+    /// The length of the slice is never checked.
+    pub unsafe fn from_slice_start_unchecked(src: &[u16]) -> (Self,usize) {
+        let first = *src.get_unchecked(0);
+        if first.is_utf16_leading_surrogate() {
+            (Utf16Char{ units: [first, *src.get_unchecked(1)] }, 2)
+        } else {
+            (Utf16Char{ units: [first, 0] }, 1)
+        }
+    }
     /// Validate and store a UTF-16 pair as returned from `char.to_utf16_tuple()`.
     pub fn from_tuple(utf16: (u16,Option<u16>)) -> Result<Self,InvalidUtf16Tuple> {
-        char::from_utf16_tuple(utf16).map(|_|
-            Utf16Char{ units: [utf16.0, utf16.1.unwrap_or(0)] }
-        )
+        unsafe {char::from_utf16_tuple(utf16).map(|_|
+            Self::from_tuple_unchecked(utf16)
+        )}
+    }
+    /// Create an `Utf16Char` from a tuple as returned from `char.to_utf16_tuple()`.
+    ///
+    /// # Safety
+    /// The units must represent a single valid codepoint.
+    pub unsafe fn from_tuple_unchecked(utf16: (u16,Option<u16>)) -> Self {
+        Utf16Char{ units: [utf16.0, utf16.1.unwrap_or(0)] }
     }
 
     /// Returns 1 or 2.
     /// There is no `.is_emty()` because it would always return false.
     pub fn len(self) -> usize {
-        if self.units[1] == 0 {1} else {2}
+        1 + (self.units[1] as usize >> 15)
+    }
+    /// Is this codepoint an ASCII character?
+    #[cfg(not(feature="std"))]
+    pub fn is_ascii(&self) -> bool {
+        self.units[0] <= 127
     }
 
     /// Convert from UTF-16 to UTF-32
@@ -208,8 +294,4 @@ impl Utf16Char {
     pub fn to_tuple(self) -> (u16,Option<u16>) {
         (self.units[0],  if self.len()==2 {Some(self.units[1])} else {None})
     }
-    #[cfg(feature="no_std")]
-    pub fn is_ascii(&self) -> bool {
-        self.units[0] <= 127
-    }
 }
