@@ -11,12 +11,15 @@ use error::{InvalidUtf8Slice,InvalidUtf8Array};
 use Utf8Iterator;
 use CharExt;
 use U8UtfExt;
+use Utf16Char;
 extern crate core;
-use self::core::{hash, fmt, str};
+use self::core::{hash, fmt, str, ptr};
 use self::core::borrow::Borrow;
 use self::core::ops::Deref;
 use self::core::mem::transmute;
-#[cfg(not(feature="no_std"))]
+#[cfg(feature="std")]
+use self::core::iter::FromIterator;
+#[cfg(feature="std")]
 use std::ascii::AsciiExt;
 #[cfg(feature="ascii")]
 extern crate ascii;
@@ -35,10 +38,9 @@ use self::ascii::{AsciiChar,ToAsciiChar,ToAsciiCharError};
 #[derive(Clone,Copy)]
 
 
-/// Store a `char` as UTF-8 so it can be borrowed as a `str`
+/// An unicode codepoint stored as UTF-8.
 ///
-/// Has the same size as `char`, and is a `[u8;4]`
-/// with the invariant that the first nth bytes are valid UTF-8, and the remaining are zero.
+/// It can be borrowed as a `str`, and has the same size as `char`.
 pub struct Utf8Char {
     bytes: [u8; 4],
 }
@@ -62,6 +64,35 @@ impl str::FromStr for Utf8Char {
         }
     }
 }
+impl From<Utf16Char> for Utf8Char {
+    fn from(utf16: Utf16Char) -> Utf8Char {
+        match utf16.to_tuple() {
+            (a @ 0...0x00_7f, _) => {
+                Utf8Char{ bytes: [a as u8, 0, 0, 0] }
+            },
+            (u @ 0...0x07_ff, _) => {
+                let b = 0x80 |  (u & 0x00_3f) as u8;
+                let a = 0xc0 | ((u & 0x07_c0) >> 6) as u8;
+                Utf8Char{ bytes: [a, b, 0, 0] }
+            },
+            (u, None) => {
+                let c = 0x80 |  (u & 0x00_3f) as u8;
+                let b = 0x80 | ((u & 0x0f_c0) >> 6) as u8;
+                let a = 0xe0 | ((u & 0xf0_00) >> 12) as u8;
+                Utf8Char{ bytes: [a, b, c, 0] }
+            },
+            (f, Some(s)) => {
+                let f = f + (0x01_00_00u32 >> 10) as u16;
+                let d = 0x80 |  (s & 0x00_3f) as u8;
+                let c = 0x80 | ((s & 0x03_c0) >> 6) as u8
+                             | ((f & 0x00_03) << 4) as u8;
+                let b = 0x80 | ((f & 0x00_fc) >> 2) as u8;
+                let a = 0xf0 | ((f & 0x07_00) >> 8) as u8;
+                Utf8Char{ bytes: [a, b, c, d] }
+            }
+        }
+    }
+}
 impl From<char> for Utf8Char {
     fn from(c: char) -> Self {
         Utf8Char{ bytes: c.to_utf8_array().0 }
@@ -80,6 +111,38 @@ impl IntoIterator for Utf8Char {
         Utf8Iterator::from(self)
     }
 }
+#[cfg(feature="std")]
+impl Extend<Utf8Char> for Vec<u8> {
+    fn extend<I:IntoIterator<Item=Utf8Char>>(&mut self, iter: I) {
+        let iter = iter.into_iter();
+        self.reserve(iter.size_hint().0);
+        for u8c in iter {
+            self.extend_from_slice(u8c.as_bytes());
+        }
+    }
+}
+#[cfg(feature="std")]
+impl Extend<Utf8Char> for String {
+    fn extend<I:IntoIterator<Item=Utf8Char>>(&mut self, iter: I) {
+        unsafe { self.as_mut_vec().extend(iter) }
+    }
+}
+#[cfg(feature="std")]
+impl FromIterator<Utf8Char> for Vec<u8> {
+    fn from_iter<I:IntoIterator<Item=Utf8Char>>(iter: I) -> Self {
+        let mut vec = Vec::new();
+        vec.extend(iter);
+        return vec;
+    }
+}
+#[cfg(feature="std")]
+impl FromIterator<Utf8Char> for String {
+    fn from_iter<I:IntoIterator<Item=Utf8Char>>(iter: I) -> Self {
+        let mut string = String::new();
+        string.extend(iter);
+        return string;
+    }
+}
 
 
   /////////////////
@@ -116,42 +179,43 @@ impl Deref for Utf8Char {
   ////////////////
  //ascii traits//
 ////////////////
-#[cfg(not(feature="no_std"))]
+#[cfg(feature="std")]
 impl AsciiExt for Utf8Char {
     type Owned = Utf8Char;
     fn is_ascii(&self) -> bool {
         self.bytes[0].is_ascii()
     }
     fn eq_ignore_ascii_case(&self,  other: &Self) -> bool {
-        self.to_char().eq_ignore_ascii_case(&other.to_char())
+        if self.is_ascii() {self.bytes[0].eq_ignore_ascii_case(&other.bytes[0])}
+        else               {self == other}
     }
     fn to_ascii_uppercase(&self) -> Self::Owned {
-        let ascii = self.bytes[0].to_ascii_uppercase();
-        if ascii == self.bytes[0] {*self}
-        else {Utf8Char{ bytes: [ascii,0,0,0] }}
+        let mut uc = *self;
+        uc.make_ascii_uppercase();
+        uc
     }
     fn to_ascii_lowercase(&self) -> Self::Owned {
-        let ascii = self.bytes[0].to_ascii_lowercase();
-        if ascii == self.bytes[0] {*self}// unchanged
-        else {Utf8Char{ bytes: [ascii,0,0,0] }}// is ascii
+        let mut uc = *self;
+        uc.make_ascii_lowercase();
+        uc
     }
     fn make_ascii_uppercase(&mut self) {
-        *self = self.to_ascii_uppercase()
+        self.bytes[0].make_ascii_uppercase()
     }
     fn make_ascii_lowercase(&mut self) {
-        *self = self.to_ascii_lowercase();
+        self.bytes[0].make_ascii_lowercase();
     }
 }
 
 #[cfg(feature="ascii")]
-/// Requires feature "ascii".
+/// Requires the feature "ascii".
 impl From<AsciiChar> for Utf8Char {
     fn from(ac: AsciiChar) -> Self {
         Utf8Char{ bytes: [ac.as_byte(),0,0,0] }
     }
 }
 #[cfg(feature="ascii")]
-/// Requires feature "asciiChar".
+/// Requires the feature "ascii".
 impl ToAsciiChar for Utf8Char {
     fn to_ascii_char(self) -> Result<AsciiChar, ToAsciiCharError> {
         self.bytes[0].to_ascii_char()
@@ -187,26 +251,49 @@ impl Utf8Char {
     /// If it's a str and you know it contains only one codepoint,
     /// use `.from_str()` to skip the validation.
     pub fn from_slice_start(src: &[u8]) -> Result<(Self,usize),InvalidUtf8Slice> {
-        // Converts to char to check codepoint, but we don't need the char.
-        let len = try!(char::from_utf8_slice(src)).1;
+        char::from_utf8_slice_start(src).map(|(_,len)| {
+            let mut bytes = [0; 4];
+            bytes[..len].copy_from_slice(&src[..len]);
+            (Utf8Char{bytes: bytes}, len)
+        })
+    }
+    /// A `from_slice_start()` that doesn't validate the codepoint.
+    ///
+    /// # Safety
+    /// The slice must be non-empty and start with a valid UTF-8 codepoint.  
+    /// Invalid or incomplete values might cause buffer overflows and overreads.
+    pub unsafe fn from_slice_start_unchecked(src: &[u8]) -> (Self,usize) {
+        let len = 1+src.get_unchecked(0).extra_utf8_bytes_unchecked();
         let mut bytes = [0; 4];
-        bytes[..len].copy_from_slice(&src[..len]);
-        Ok((Utf8Char{bytes: bytes}, len))
+        ptr::copy_nonoverlapping(src.as_ptr(), &mut bytes[0] as *mut u8, len);
+        (Utf8Char{ bytes: bytes }, len)
     }
     /// Validate the array and store it.
     pub fn from_array(utf8: [u8;4]) -> Result<Self,InvalidUtf8Array> {
-        try!(char::from_utf8_array(utf8));
-        let extra = utf8[0].extra_utf8_bytes_unchecked() as u32;
-        let mask = u32::from_le(0xff_ff_ff_ff >> 8*(3-extra));
-        let unused_zeroed = mask  &  unsafe{ transmute::<_,u32>(utf8) };
-        Ok(unsafe{ transmute(unused_zeroed) })
+        unsafe {
+            try!(char::from_utf8_array(utf8));
+            let extra = utf8[0].extra_utf8_bytes_unchecked() as u32;
+            let mask = u32::from_le(0xff_ff_ff_ff >> 8*(3-extra));
+            let unused_zeroed = mask  &  transmute::<_,u32>(utf8);
+            Ok(transmute(unused_zeroed))
+        }
+    }
+    /// Unused bytes must be zero
+    pub unsafe fn from_array_unchecked(utf8: [u8;4]) -> Self {
+        Utf8Char{ bytes: utf8 }
     }
 
-    /// Result is 1...4 and identical to `.as_ref().len()` or `.as_char().len_utf8()`.
-    /// There is no .is_emty() because it would always return false.
+    /// Result is 1...4 and identical to `.as_ref().len()` or
+    /// `.as_char().len_utf8()`.
+    /// There is no .is_emty() because this type is never empty.
     pub fn len(self) -> usize {
         self.bytes[0].extra_utf8_bytes_unchecked() + 1
     }
+    /// Is this codepoint an ASCII character?
+    #[cfg(not(feature="std"))]
+    pub fn is_ascii(&self) -> bool {
+        self.bytes[0] <= 127
+    }
 
     /// Convert from UTF-8 to UTF-32
     pub fn to_char(self) -> char {
@@ -230,8 +317,9 @@ impl Utf8Char {
     pub fn to_array(self) -> ([u8;4],usize) {
         (self.bytes, self.len())
     }
-    #[cfg(feature="no_std")]
-    pub fn is_ascii(&self) -> bool {
-        self.bytes[0] <= 127
+    /// Return a `str` view of the array the codepoint is stored as.
+    /// Ns an unambiguous version of `.as_ref()`.
+    pub fn as_str(&self) -> &str {
+        self.deref()
     }
 }
