@@ -10,15 +10,14 @@ use Utf8Char;
 extern crate core;
 use self::core::{mem, u32, u64};
 use self::core::ops::Not;
-#[cfg(not(feature="no_std"))]
 use self::core::fmt;
-#[cfg(not(feature="no_std"))]
+#[cfg(feature="std")]
 use std::io::{Read, Error as ioError};
 
 
 
-/// Read or iterate over the bytes in the UTF-8 representation of a codepoint.
-#[derive(Clone,Copy)]
+/// Read or iterate over the bytes of the UTF-8 representation of a codepoint.
+#[derive(Clone)]
 pub struct Utf8Iterator (u32);
 
 impl From<Utf8Char> for Utf8Iterator {
@@ -55,7 +54,7 @@ impl ExactSizeIterator for Utf8Iterator {
         4 - unused_bytes as usize
     }
 }
-#[cfg(not(feature="no_std"))]
+#[cfg(feature="std")]
 impl Read for Utf8Iterator {
     /// Always returns Ok
     fn read(&mut self,  buf: &mut[u8]) -> Result<usize, ioError> {
@@ -69,10 +68,14 @@ impl Read for Utf8Iterator {
         Ok(buf.len())
     }
 }
-#[cfg(not(feature="no_std"))]
 impl fmt::Debug for Utf8Iterator {
     fn fmt(&self,  fmtr: &mut fmt::Formatter) -> fmt::Result {
-        let content: Vec<u8> = self.collect();
-        write!(fmtr, "bytes left: {:?}, content: {:x}", content, self.0)
+        let mut content = [0; 4];
+        let mut i = 0;
+        for b in self.clone() {
+            content[i] = b;
+            i += 1;
+        }
+        write!(fmtr, "{:?}", &content[..i])
     }
 }
